/*****************************************************************************
*
*     Revision List:
*
*       Ver     Date        Who     Description
*       ---------------------------------------------------------------------
*       X1.00   03.07.31    GJO     Wrote program

******************************************************************************/

//*****************************************************************************
//* Required header files
//*****************************************************************************

#include "16F877A.h"    // Device header file
#device *=16 ADC=10     // Use 16 bit pointers and 10 bit ADC mode.
#include <string.h>
#include <stdlib.h>
#include <stdio.h>



/******************************************************************************/
/*Setup for delay, serial port, device fuses, and interrupt priority          */
/******************************************************************************/
#use delay(clock=16000000, RESTART_WDT)

#use rs232(baud=57600,xmit=PIN_C6,rcv=PIN_C7,RESTART_WDT)

#fuses HS, NOPROTECT, WDT, PUT, BROWNOUT, NOLVP

#priority TIMER0, RDA, TIMER2, AD, EXT

#include "Term.c"

/******************************************************************************/
/*PIC configuration register addresses                                        */
/******************************************************************************/

#byte PORTB = 0x06      // Define the address of the I/O
#byte PORTC = 0x07      // ports for direct reading and writing
#byte PORTD = 0x08      // ports for direct reading and writing

#bit    CREN    = 0x18.4    //Clear errors bit
#bit    GO_DONE = 0x1F.2    //ADC start conversion bit
long    TMR1;               //Variable for accessing Timer 1 count register
#byte   TMR1    = 0x0E
#byte   TMR1L   = 0x0E      //Timer 1 count register low byte
#byte   TMR1H   = 0x0F      //Timer 1 count register high byte
#byte   CCPR1L  = 0x15      //PWM1 register 1
#byte   CCPR1H  = 0x16      //PWM1 register 2
#byte   CCP1CON = 0x17      //CCP1 control register
#byte   ADCON0  = 0x1F      //ADC control register 0
#byte   ADCON1  = 0x9F      //ADC control register 1
#byte   ADRESH  = 0x1E      //ADC result high byte register
#byte   ADRESL  = 0x9E      //ADC result low byte register
#byte   RCSTA   = 0x18      //Defines Receive status port

#word ADRES = ADRESL

/******************************************************************************/
/*PIC A/D configuration register setup values (Pic Manual Page 111-112)       */
/******************************************************************************/
#define AD_SETUP_0  0b10000001  //ADCON0 setup byte (Fosc/32)
#define AD_SETUP_1  0b10000001  //ADCON1 setup byte (VREF=RA3, 7 Analogs)

/******************************************************************************/
/*Prevent the compiler from programming TRIS registers when IO pins are used  */
/******************************************************************************/
#use fast_io(a)
#use fast_io(b)
#use fast_io(c)
#use fast_io(d)
#use fast_io(e)

/******************************************************************************/
/*IO port direction settings                                                  */
/******************************************************************************/
#define PORT_A_MASK 0b11111111      //Port A direction register mask
#define PORT_B_MASK 0b01110000      //Port B direction register mask
#define PORT_C_MASK 0b10010000      //Port C direction register mask
#define PORT_D_MASK 0b11110000      //Port D direction register mask
#define PORT_E_MASK 0b00000111      //Port E direction register mask

/******************************************************************************/
/*IO pin definitions                                                          */
/******************************************************************************/

// *********************** Port B *************************************

#define ND1             PIN_B0  //
#define ND2             PIN_B1  //
#define ND3             PIN_B2  //
#define ND4             PIN_B3  //
#define RADIO_INPUT     PIN_B4  //
#define ND5             PIN_B5  //
#define ND6             PIN_B6  //
#define ND8             PIN_B7  //

//************************ Port C *************************************

#define ND9             PIN_C0  //
#define ND10            PIN_C1  //
#define ND11            PIN_C2  //
#define ND12            PIN_C3  //
#define ND13            PIN_C4  //
#define ND14            PIN_C5  //
#define TXD             PIN_C6  //
#define RXD             PIN_C7  //

//************************ Port D *************************************

#define ND17            PIN_D0  //
#define ND18            PIN_D1  //
#define ND19            PIN_D2  //
#define ND20            PIN_D3  //
#define ND21            PIN_D4  //
#define ND22            PIN_D5  //
#define ND23            PIN_D6  //
#define ND24            PIN_D7  //

/******************************************************************************/
/* Defines                                                            */
/******************************************************************************/


/******************************************************************************/
/*Useful macros                                                            */
/******************************************************************************/



/******************************************************************************/
/*Global variables                                                            */
/******************************************************************************/
/*
typedef struct {
	int preamble;
	int sequence_tag;
	int source_address;
	int payload[10];
	long frame_check_word;
	int end_byte;
} radio_packet;

radio_packet rp;
*/
char rbits;
char rbyte_count;

#define PACKET_SIZE 8
#define NUM_BITS PACKET_SIZE * 8


unsigned char buffer[PACKET_SIZE];  // working buffer for code being received
unsigned char buffer1[PACKET_SIZE];  //
unsigned char byte_count = 0;
unsigned char screen_update_not_busy = TRUE;

unsigned long rssi=0, rssitemp=0;


//enum { WAIT_SYNC, GET_BIT, GET_PAUSE };

//*****************************************************************************
// Flags
//*****************************************************************************



/******************************************************************************/
/* ROM constants - IMPORTANT: must be declared after variables                */
/******************************************************************************/


/******************************************************************************/
/*Function prototypes                                                         */
/******************************************************************************/


void system_100us_timer (void);
void system_5ms_timer (void);
void Initialize(void);


//*****************************************************************************
//* Custom SCU Compile Options
//*****************************************************************************


//*****************************************************************************
//*********************** BEGIN OF FUNCTIONS **********************************
//*****************************************************************************

typedef unsigned int T_RADIO_DATA;


void radio_initialize( void );
void radio_poll( void );
int32 radio_get_packet( void );
unsigned int radio_to_ID( unsigned int32 raw );
unsigned char radio_is_trigger( unsigned int32 raw );
unsigned char radio_is_battery( unsigned int32 raw );
unsigned char radio_is_test( unsigned int32 raw );

long sum;

// RF packet bit fields:
//---------------------------------------------
// START    1 bit       must be always 1
// ID       7 bits      identifies trap
// TRIGGER  1 bit       trap is triggered
// BATTERY  1 bit       trap low battery flag
// TEST     1 bit       trap is in test/reduced power mode
//
//---------------------------------------------
//
// RF waveform encoding:
//
//
//
//        logic 1 = .66 mS .33 mS
//                  +-----+--+
//
//
//                logic 0 = .33 mS .66 mS
//                           +--+-----+
//
//               +--+     +--+  +--+--+  +--+     +--
//               |  |     |  |  |     |  |  |     |
//----....-------+  +--+--+  +--+     +--+  +--+--+
//
//
//  sync pause   start    1' bit   2'bit    3'bit     ... total: 10 bits + start
//(min 11.52 mS)


//min and max pulse duration, according to the above waveform sampled @ 10 kHz
//
//#define SAMPLE100K

#ifdef SAMPLE100K

#define MAX_ONE 4
#define MIN_ONE 2
#define MAX_ZERO 8
#define MIN_ZERO 5
#define MIN_PAUSE MIN_ONE
#define MAX_PAUSE MAX_ZERO
#define MIN_SYNC 110
#define RADIO_SAMPLING_FREQ 211
#define RX_INTERVALL 200
#define TO_LIMIT (NUM_BITS * 10) + 50

#else

#define MAX_ONE 8
#define MIN_ONE 4
#define MAX_ZERO 16
#define MIN_ZERO 10
#define MIN_PAUSE MIN_ONE
#define MAX_PAUSE MAX_ZERO
#define MIN_SYNC 220
#define RADIO_SAMPLING_FREQ 234
#define RX_INTERVALL 200
#define TO_LIMIT (NUM_BITS * 20) + 100

#endif


int tx_count = 0;

int32 received = 0;      //buffer for last valid data recived

short RX_TRIGGER_FLAG = 0;

///////////////////////////////////////////////////////////////////////////////
// radio_initialize - initialize receiver I/O                                //
///////////////////////////////////////////////////////////////////////////////

void radio_initialize(void)
{
    //RADIO_PULLUP = 1;                   //turn on pullups
    //RADIO_DDR    = 0;                   //and set port direction to input
}



///////////////////////////////////////////////////////////////////////////////
// radio_poll - samples the receiver input and decodes incoming data         //
///////////////////////////////////////////////////////////////////////////////


void radio_poll( void )
{
    enum { WAIT_SYNC, GET_BIT, GET_PAUSE };                         //radio decoder states
    static unsigned char mode  = WAIT_SYNC;                         //current decoder state
    static unsigned char width = 0;                                 //measures pulse width
    static unsigned char bits = 0;                                  //counts bit received

    static long to_ctr=0, adc_temp=0;


	char i;

    switch ( mode )
    {
    case WAIT_SYNC:
        if (input(RADIO_INPUT) == 0)                                //is pause end?
            width++;                                                //no, keep measuring pause width
        else if (width > MIN_SYNC)                                  //pause ended: was it long enough
        {                                                           //to qualify for a SYNC pause?
			// Start ADC conversion here
            //GO_DONE = 1;
            byte_count = 0;
            bits = 0;
            rbyte_count = 0;
            rbits = 0;
            memset(buffer, 0x00, sizeof(buffer));                 // yes, clear buffers and prepare
            mode = GET_BIT;                                         //to receive first bit of sequence
            width = 0;
            to_ctr=0;
        }
        else
            width = 0;                                              //otherwise keep loking for
        break;                                                      //a SYNC pause

    case GET_BIT:
        if (input(RADIO_INPUT) == 1)                                       //is pulse end?
            width++;                                                //no, keep measuring pulse width
        else
        {                                                           //otherwise check if it was a well-shaped pulse
            if ( (width >= MIN_ONE) && (width <= MAX_ONE) )         //is pulse width within limits for "1"?
            {
                buffer[byte_count] = (buffer[byte_count] << 1) | 1;   //yes, got a valid "1"
                bits++;                                                 //shift it in, count it,
                if( 0 == (bits % 8)) byte_count++;
                mode = GET_PAUSE;                                       //and prepare for a pause
            }
            else if ( (width >= MIN_ZERO) && (width <= MAX_ZERO) )      //is pulse width within limits for "0"?
            {
                buffer[byte_count] = (buffer[byte_count] << 1);       //yes, got a valid "0"
                bits++;                                                 //shift it in, count it
                if(0 == (bits % 8)) byte_count++;

                mode = GET_PAUSE;                                       //and prepare for a pause
            }
            else
            {
                mode = WAIT_SYNC;                                   //width is neither a valid one or zero
            };                                                      //therefore abort reception

            rbits = bits;
            rbyte_count = byte_count;

			if(bits == NUM_BITS /2 ) GO_DONE = 1;

            if ((bits == NUM_BITS))         //got all bits?
            {
                // Verify Checksum of received packet
                sum=0;
                // Adapted to 8-bit from RFC-1171
                for(i=0; i < sizeof(buffer); i++)
                {
                    sum += (long) buffer[i];
                }

                // Fold 16-bit sum to 8 bits
                while (sum >> 8)
                {
                    sum = (sum & 0xFF) + (sum >> 8);
                }
                sum = (~sum & 0xFF);

                if((RX_TRIGGER_FLAG == FALSE))
                {
                    memcpy(buffer1, buffer , sizeof(buffer));
                    //rssi = rssitemp;
				}
                RX_TRIGGER_FLAG = TRUE;
            }
            else if (bits == 1)                                     //first bit?
            {
                if((buffer[0] & 1) != 1 )                          //yes, then it must be "1"
                {
                    mode = WAIT_SYNC;                               //otherwise abort reception
                };                                                  //and wait for a new sync
            };
            width = 0;
        };
        break;

    case GET_PAUSE:
        if (input(RADIO_INPUT) == 0)                                       //is pause end?
            width++;                                                //no, keep meausring pause width
        else
        {                                                           //otherwise check if it was of correct duration
            if ( (width >= MIN_PAUSE) && (width <= MAX_PAUSE) )     //is width within pause limits?
            {
                mode = GET_BIT;                                     //yes, got a valid pause
                width = 0;                                          //prepare to receive a new bit
            }
            else
            {
                mode = WAIT_SYNC;                                   //otherwise reject abort reception
                width = 0;                                          //and wait for a new sync
            };
        };
        break;

    }; // switch(mode)

    if (width == 255)                                               //prevent "width" from overflows
            width--;                                                //that could result in false readings
}



///////////////////////////////////////////////////////////////////////////////
// radio_get_packet- returns the last data received and clears               //
// the receive buffer                                                        //
///////////////////////////////////////////////////////////////////////////////

int32 radio_get_packet( void )
{
    unsigned int32 retval;

/*  "received" cannot be passed diretcly to main program,
    because it is modified by an ISR
    to touch it, disable interrupt and use a buffer
*/
    //disable_interrupts(INT_TIMER0);
    //memcpy(rx_buffer, buffer3 , sizeof(rx_buffer));
    //memset(buffer3, 0x00, sizeof(buffer3));
    //enable_interrupts(INT_TIMER0);
    return retval;
}





//******************************************************************************
// Function:    ServiceTimer1Interrupt
//*****************************************************************************

#int_timer0
void system_100us_timer (void)
{
    output_high(ND8);
    set_timer0(RADIO_SAMPLING_FREQ); // Re-load timer for 100us overflow
    radio_poll();
    output_low(ND8);
}

//******************************************************************************
// Function:    ServiceTimer1Interrupt
//*****************************************************************************

#int_timer2
void system_5ms_timer (void)
{
    Read_ADC(ADC_START_ONLY);
    if(tx_count++ == RX_INTERVALL){
        //RX_TRIGGER_FLAG = TRUE;
        tx_count = 0;
    }
}


#int_AD
void read_ADC_result()
{
	// read RSSI voltage on AN0
	rssi =  Read_ADC();
}

/******************************************************************************/
/*Function:     Initialize                                                    */
/*Description:  Initializes PIC setup                                         */
/*Parameters:   None                                                          */
/*Return Value: None                                                          */
/******************************************************************************/
//#separate
void Initialize(void)
{
    set_tris_a(PORT_A_MASK);                        //Setup IO direction registers as defined
    set_tris_b(PORT_B_MASK);
    set_tris_c(PORT_C_MASK);
    set_tris_d(PORT_D_MASK);
    set_tris_e(PORT_E_MASK);

    PORTB = 0b00000000;                             // Initialize port pins
    PORTC = 0b11000100;
    PORTD = 0b00000011;                             //

    ADCON0 = AD_SETUP_0;                            //Setup ADC (SCU_R??.H)
    ADCON1 = AD_SETUP_1;                            //Setup ADC ports

    set_uart_speed(57600);                           // Setup the on board UART

    setup_counters(RTCC_INTERNAL, WDT_2304MS);      // Set watch dog timer for 2304ms.

    setup_timer_0(RTCC_DIV_8 | RTCC_INTERNAL);      // 333us tick for TX data clock

    setup_timer_1(T1_INTERNAL);                     //

    setup_timer_2(T2_DIV_BY_16, 249, 5);            // 5ms tick


	setup_port_a( AN0_AN1_AN3 );
	setup_adc( ADC_CLOCK_INTERNAL );
   	set_adc_channel( 0 );


     // Initialize all timers
    set_timer0(RADIO_SAMPLING_FREQ);                //
    set_timer1(0);
    set_timer2(0);

    //Enable interrupts
    enable_interrupts(INT_TIMER0);                  // 333us timer
    //enable_interrupts(INT_TIMER2);                  // 10ms timer
    enable_interrupts(INT_RDA);                     // RDA (USART Receive).
    enable_interrupts(INT_AD);                      // ADC interrupt
 //   enable_interrupts(GLOBAL);                      // Enable interrupt
}


//*****************************************************************************
// main():
// Description: This is the starting point for the PIC on startup or after
//              reset.
//*****************************************************************************

void main(void)
{

    //long sum;

    char i;
    unsigned char trigger   = FALSE;  //flag, trap triggered
    unsigned char low_bat   = FALSE;  //flag, time to change battery
    unsigned int  interval  = 0;      //time (in wake-up timer ticks) between
                                      //"heartbeat" transmissions
    unsigned char ID = 100;           //copy non-volatile value from FLASH



    long radio_data = 0;
    long temp, temp2, volts2;

    int32 volts, u32temp;

    signed int kty81;

    Initialize();                     // Initialize PIC Hardware

    enable_interrupts(GLOBAL);                      // Enable interrupt

    output_high(ND8);
    delay_ms(1000);
    output_low(ND8);

	Display_attributes(FORE_BLUE);

	Screen_clear();
	Screen_goto(1,1);
    printf("RXM-418-LR - V4.01, 16F877A: ");
    printf(__DATE__);
	printf("\nDATA LINK DIAGNOSTIC UTILITY:");
	printf("\n-----------------------------");

    enable_interrupts(GLOBAL);               // Enable interrupt

    for( ; ; )                              // Main Processing Loop.
    {
        restart_wdt();                      // Stroke the watchdog

        if(RX_TRIGGER_FLAG)
        {
            Display_attributes(FORE_RED);
            Screen_goto(4,1);
            // Verify checksum
            if(sum == 0)
                printf("CHECKSUM    = OK  ");
            else
                printf("CHECKSUM    = FAIL");

            Screen_goto(5,1);
            printf("BYTE COUNT  = %u ", rbyte_count);
            printf("\nNUM BITS  = %u ", rbits);
            printf("\nRAW PACKET  = ");

            for (i=0; i< sizeof(buffer1); i++)
                printf("%02X ", buffer1[i]);

			Screen_goto(8,1);
			Display_attributes(FORE_BLUE);

			if(rssi > 454) rssi = 454;

			if(rssi < 260) rssi = 260;

			temp2 = rssi - 260;

			temp2 = (int32) temp2 * 100 / 184;

			printf("RSSI        = [%lu%%] ", temp2);
			// Bargraph
			for(i=0;i<temp2 / 2 ;i++)
			{
				printf("|");
			}

			for(i=0;i < 50 - (temp2 / 2) ;i++)
			{
				printf(" ");
			}
			Display_attributes(FORE_BLACK);

            Screen_goto(10,1);
            if(temp > 600){
                printf("BATTERY     = OK ");
            }else{
                printf("BATTERY     = LOW ");
            }

            printf("\r\nSF BITS     = %u ",buffer1[0] & 0x70 );
            printf("\r\nPACKET NO.  = %u ",buffer1[0] & 0x0F );
            printf("\r\nTX ADDR     = %u ",buffer1[1] & 0x0F );
            printf("\r\nS3          = %u ",(buffer1[2] & 0x80) ? 1 : 0 );
            printf("\r\nS2          = %u ",(buffer1[2] & 0x40) ? 1 : 0 );
            printf("\r\nS1          = %u ",(buffer1[2] & 0x20) ? 1 : 0 );
            printf("\r\nS0          = %u ",(buffer1[2] & 0x10) ? 1 : 0 );

            // Extract and assemble KTY81 ADC*64 value from bytes 1,5, and 6
            temp = 0;
            temp |=  ((long) (buffer1[1] & 0xF0) << 8);
            temp |=  ((long) (buffer1[5] & 0x0F) << 8);
            temp |=   (long)  buffer1[6];

            //DEG C = {[(ADC64 * 1023) / 967] / 101} - 150
            u32temp = (int32) temp * 1023;
            u32temp /= 967;

            printf("\r\nSENSOR TEMP = %01.1f C  ", ((float) u32temp / 101.0) - 150.0 );

            volts2 = 0;
            volts2 = (buffer1[5] & 0xF0) >> 4; // Lower part of LSB nibble
            volts2 |= (long) buffer1[4] << 4;
            // VOLTS = (ADC * VREF * 6) / 1000, where VREF = 1.780V
            printf("\r\nEXT VOLTS   = %01.2f ", (float) volts2 * 0.0104399 );

            temp =  make16(buffer1[2] & 0x03, buffer1[3]);
            volts = (int32) temp * 235 / 64;
            printf("\r\nBATT. VOLTS = %01.2f ", ((float) volts) / 1000.0 );
            RX_TRIGGER_FLAG = FALSE;
            Screen_goto(3,1);
        }
    }
}

// ******************************************************************************
// ******************************************* end of main **********************
//*******************************************************************************




